###
# @copyright Copyright (C) 2022 Spyrosoft Solutions. All rights reserved.
# THIS FILE WAS GENERATED AUTOMATICALLY. DO NOT CHANGE IT.
###
lizard~=1.7.9
gitpython==3.1.18
Jinja2==3.0.2
conan==1.53.0
cmake==3.24.1.1
click==8.1.3
toml==0.10.2
pyserial==3.5
lief==0.12.3
shellingham==1.5.0
